import 'dart:io';

class BuildWithAmr extends _JsonConverter {
  BuildWithAmr(json, {required bool isNullSafety, required String modelName}) {
    print('=======? creating $modelName model');
    _json = json;
    _isNullSafety = isNullSafety ?? false;
    _fileName = modelName.getFileName();
    _modelName = modelName.getModelName();
    _builtModel();
  }
}

class _JsonConverter {
  late Map<String, dynamic> _json;
  late bool _isNullSafety;
  String _modelName = 'AmrModel';
  String _fileName = 'amr_model';
  late String _modelTemplate;
  final _imports = <String>[];

  void _builtModel() async {
    var f = File('lib/models/$_fileName.dart');
    if (await f.exists()) await f.delete();
    f.createSync(recursive: true);
    _modelTemplate =
    'class $_modelName{${_builtVariables()}\n${_builtFromJson()}\n}';
    f.writeAsStringSync(_imports.join() + _modelTemplate);
  }

  String _builtVariables() {
    var texts = <String>[];
    _json.forEach((key, value) {
      var valueType = value.runtimeType.toString();
      if (valueType.contains('List<Map<')) {
        texts.add(
            '\n${_isNullSafety ? "late" : ''} List<${key.getModelName()}> ${key
                .getVarName()};');
      } else if (!valueType.contains('_InternalLinkedHashMap')) {
        texts.add(
            '\n${_isNullSafety ? "late" : ''} $valueType ${key.getVarName()};');
      } else {
        print('hello I catch a map here');
        _imports.add("import '$key.dart';\n");
        texts.add(
            '\n${_isNullSafety ? "late" : ''} ${key.getModelName()} ${key
                .getVarName()};');
        BuildWithAmr(value, modelName: key, isNullSafety: _isNullSafety);
      }
    });
    return texts.join();
  }

  String _builtFromJson() {
    var texts = <String>[];
    _json.forEach((key, value) {
      var valueType = value.runtimeType.toString();
      if (valueType.contains('List') && !valueType.contains('Map')) {
        texts.add(
            "\n${key.getVarName()} =${_isNullSafety ? "json['$key']" ' == null? null:' : ''}List.castFrom<String, ${valueType.replaceAll('List<', '').replaceAll('>', '')}>(json['$key']);");
      } else if (!valueType.contains('_InternalLinkedHashMap')) {
        texts.add(
            "\n${key.getVarName()} = json['$key']${_isNullSafety ? "??${_getDefaultValue(value)}" : ''};");
      } else {
        texts.add(
            "\n${key.getVarName()} = ${key.getModelName()}.fromJson(json['$key']);");
      }
    });
    return '$_modelName.fromJson(Map<String,dynamic> json){${texts.join()}\n}';
  }

  dynamic _getDefaultValue(dynamic value) {
    switch (value.runtimeType) {
      case int:
        return 0;
        break;
      case String:
        return "''";
        break;
      case bool:
        return false;
      case double:
        return 0.0;
        break;
    }
  }
}

extension StringExtension on String {
  String getVarName() {
    var parts = split('_');
    var newText = <String>[];
    for (var i = 0; i < parts.length; i++) {
      var element = parts[i];
      if (i != 0) {
        element = element[0].toUpperCase() + element.substring(1);
      }
      newText.add(element);
    }
    return newText.join();
  }

  String getModelName() {
    var parts = split('_');
    var newText = <String>[];
    for (var i = 0; i < parts.length; i++) {
      var element = parts[i];
      element = element[0].toUpperCase() + element.substring(1);
      newText.add(element);
    }
    return newText.join();
  }

  String getFileName() {
    return toLowerCase();
  }
}
